﻿using System;
using System.Diagnostics;
using System.IO;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Interfaces;
using TreeSharp;

namespace FateStayTeleport
{
    public class TeleportPlugin : IBotPlugin
    {
        private Composite root;

        public void OnEnabled()
        {
            TreeRoot.OnStart += OnBotStart;
            TreeRoot.OnStop += OnBotStop;
            if (TreeRoot.IsRunning) { Hook(); }
        }

        public void OnDisabled()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            Unhook();
        }

        private void OnBotStart(BotBase bot)
        {
            Hook();
        }

        private void OnBotStop(BotBase bot)
        {
            Unhook();
        }

        private void Hook()
        {
            var settings = DataLoader.YamlLoad<TeleportSettings>(Directories.SETTINGS) ?? new TeleportSettings();
            root = new TeleportLogic(settings).GetBehavior();
            TreeHooks.Instance.InsertHook("TreeStart", 0, root);
        }

        private void Unhook()
        {
            if (root == null) { return; }
            TreeHooks.Instance.RemoveHook("TreeStart", root);
            root = null;
        }

        #region PluginInfo
        public void OnPulse()
        {

        }

        public void OnInitialize()
        {

        }

        public void OnShutdown()
        {

        }

        public string Author
        {
            get { return "Saga"; }
        }

        public Version Version
        {
            get { return new Version(0, 1); }
        }

        public string Name
        {
            get { return "FateStayTeleport"; }
        }

        public string Description
        {
            get { return string.Empty; }
        }

        public bool WantButton
        {
            get { return true; }
        }

        public string ButtonText
        {
            get { return "Settings"; }
        }

        public void OnButtonPress()
        {
            if (!File.Exists(Directories.SETTINGS))
            {
                var settings = new TeleportSettings();
                DataLoader.YamlSave(Directories.SETTINGS, settings);
            }

            Process.Start("notepad.exe", Directories.SETTINGS);
        }

        public bool Equals(IBotPlugin other)
        {
            return false;
        }
        #endregion
    }
}
