//  This file is part of YamlDotNet - A .NET library for YAML.
//  Copyright (c) Antoine Aubry and contributors
    
//  Permission is hereby granted, free of charge, to any person obtaining a copy of
//  this software and associated documentation files (the "Software"), to deal in
//  the Software without restriction, including without limitation the rights to
//  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
//  of the Software, and to permit persons to whom the Software is furnished to do
//  so, subject to the following conditions:
    
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
    
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.

using System;
using System.Collections.Generic;

namespace YamlDotNet.Core
{
	/// <summary>
	/// Generic queue on which items may be inserted
	/// </summary>
	[Serializable]
	public class InsertionQueue<T>
	{
		// TODO: Use a more efficient data structure

		private readonly IList<T> items = new List<T>();

		/// <summary>
		/// Gets the number of items that are contained by the queue.
		/// </summary>
		public int Count
		{
			get
			{
				return items.Count;
			}
		}

		/// <summary>
		/// Enqueues the specified item.
		/// </summary>
		/// <param name="item">The item to be enqueued.</param>
		public void Enqueue(T item)
		{
			items.Add(item);
		}

		/// <summary>
		/// Dequeues an item.
		/// </summary>
		/// <returns>Returns the item that been dequeued.</returns>
		public T Dequeue()
		{
			if (Count == 0)
			{
				throw new InvalidOperationException("The queue is empty");
			}

			T item = items[0];
			items.RemoveAt(0);
			return item;
		}

		/// <summary>
		/// Inserts an item at the specified index.
		/// </summary>
		/// <param name="index">The index where to insert the item.</param>
		/// <param name="item">The item to be inserted.</param>
		public void Insert(int index, T item)
		{
			items.Insert(index, item);
		}
	}
}