//  This file is part of YamlDotNet - A .NET library for YAML.
//  Copyright (c) Antoine Aubry and contributors
    
//  Permission is hereby granted, free of charge, to any person obtaining a copy of
//  this software and associated documentation files (the "Software"), to deal in
//  the Software without restriction, including without limitation the rights to
//  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
//  of the Software, and to permit persons to whom the Software is furnished to do
//  so, subject to the following conditions:
    
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
    
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.

using System.Collections.Generic;
using System.Collections.ObjectModel;
using YamlDotNet.Core.Tokens;

namespace YamlDotNet.Core
{
	/// <summary>
	/// Collection of <see cref="TagDirective"/>.
	/// </summary>
	public class TagDirectiveCollection : KeyedCollection<string, TagDirective>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="TagDirectiveCollection"/> class.
		/// </summary>
		public TagDirectiveCollection()
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="TagDirectiveCollection"/> class.
		/// </summary>
		/// <param name="tagDirectives">Initial content of the collection.</param>
		public TagDirectiveCollection(IEnumerable<TagDirective> tagDirectives)
		{
			foreach (var tagDirective in tagDirectives)
			{
				Add(tagDirective);
			}
		}

		/// <summary/>
		protected override string GetKeyForItem(TagDirective item)
		{
			return item.Handle;
		}
		
		/// <summary>
		/// Gets a value indicating whether the collection contains a directive with the same handle
		/// </summary>
		public new bool Contains(TagDirective directive)
		{
			return Contains(GetKeyForItem(directive));
		}
	}
}