//  This file is part of YamlDotNet - A .NET library for YAML.
//  Copyright (c) Antoine Aubry and contributors
    
//  Permission is hereby granted, free of charge, to any person obtaining a copy of
//  this software and associated documentation files (the "Software"), to deal in
//  the Software without restriction, including without limitation the rights to
//  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
//  of the Software, and to permit persons to whom the Software is furnished to do
//  so, subject to the following conditions:
    
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
    
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.

using System;

namespace YamlDotNet.RepresentationModel
{
	/// <summary>
	/// Defines the method needed to be able to visit Yaml elements.
	/// </summary>
	public interface IYamlVisitor
	{
		/// <summary>
		/// Visits a <see cref="YamlStream"/>.
		/// </summary>
		/// <param name="stream">
		/// The <see cref="YamlStream"/> that is being visited.
		/// </param>
		void Visit(YamlStream stream);

		/// <summary>
		/// Visits a <see cref="YamlDocument"/>.
		/// </summary>
		/// <param name="document">
		/// The <see cref="YamlDocument"/> that is being visited.
		/// </param>
		void Visit(YamlDocument document);

		/// <summary>
		/// Visits a <see cref="YamlScalarNode"/>.
		/// </summary>
		/// <param name="scalar">
		/// The <see cref="YamlScalarNode"/> that is being visited.
		/// </param>
		void Visit(YamlScalarNode scalar);

		/// <summary>
		/// Visits a <see cref="YamlSequenceNode"/>.
		/// </summary>
		/// <param name="sequence">
		/// The <see cref="YamlSequenceNode"/> that is being visited.
		/// </param>
		void Visit(YamlSequenceNode sequence);

		/// <summary>
		/// Visits a <see cref="YamlMappingNode"/>.
		/// </summary>
		/// <param name="mapping">
		/// The <see cref="YamlMappingNode"/> that is being visited.
		/// </param>
		void Visit(YamlMappingNode mapping);
	}
}